channel uint inCtrlCh  __attribute__((io("aux_ctrl_in")));
channel uint outCtrlCh __attribute__((io("aux_ctrl_out")));
channel uint inDataCh  __attribute__((io("aux_data_in")));
channel uint outDataCh __attribute__((io("aux_data_out")));

#define SCCB_REQ_FLAG      (0x40000000)
#define SCCB_BUSY_FLAG     (0x80000000)
#define SCCB_WE_FLAG       (0x20000000)
#define SCCB_RD_FLAG       (0x00000000)
#define FRAME_CAPTURE_FLAG (0x10000000)

__kernel 
void ov7670_ctrl (
		  int mode, // 0: read_mode, 1: write_mode, 2: capture kick
		  int addr,
		  int din,
		  __global int * restrict dout
		  )
{
  int v = 0, r = 0;
  int state = 0;
  for(;;){
    write_channel_altera(outCtrlCh, v);
    r = read_channel_altera(inCtrlCh);
    mem_fence(CLK_CHANNEL_MEM_FENCE);
    switch(state){
    case 0:
      state = 1;
    case 1: // check status and set value
      if((r & SCCB_BUSY_FLAG) == 0){ // when not busy
	if(mode == 2){ // capture kick
	  v = FRAME_CAPTURE_FLAG;
	}else{
	  v |= (addr & 0x000000FF);
	  v |= ((din & 0x000000FF) << 8);
	  v |= (mode == 1) ? SCCB_WE_FLAG : SCCB_RD_FLAG;
	  v |= SCCB_REQ_FLAG;
	}
	state = 2;
      }
      break;
    case 2:
      if((r & SCCB_BUSY_FLAG) != 0){ // the operation has been started
	v = 0;
	state = 3;
      }
      break;
    case 3:
      if((r & SCCB_BUSY_FLAG) == 0){ // the operation has been finished
	*dout = (r & 0x00FF0000) >> 16;
	return;
      }
      break;
    default:
      break;
    }
  }
}

__kernel 
void ov7670_capture (
		   __global int * restrict b
		   )
{
  const int size = 320 * 480;
  for(int i = 0; i < size; i++){
    b[i] = read_channel_altera(inDataCh);
  }
}
